"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeepClient = void 0;
const gql_1 = require("./gql");
const minilinks_1 = require("./minilinks");
const promise_1 = require("./promise");
const reserve_1 = require("./reserve");
class DeepClient {
    constructor(options) {
        this.minilinks = options.minilinks || (0, minilinks_1.minilinks)([]);
        this.apolloClient = options.apolloClient;
        this.table = options.table || 'links';
        this.selectReturning = options.selectReturning || 'id type_id from_id to_id value';
        this.insertReturning = options.insertReturning || 'id';
        this.updateReturning = options.updateReturning || 'id';
        this.deleteReturning = options.deleteReturning || 'id';
        this.defaultSelectName = options.defaultSelectName || 'SELECT';
        this.defaultInsertName = options.defaultInsertName || 'INSERT';
        this.defaultUpdateName = options.defaultUpdateName || 'UPDATE';
        this.defaultDeleteName = options.defaultDeleteName || 'DELETE';
    }
    select(exp, options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : exp : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.selectReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultSelectName;
            const q = yield this.apolloClient.query((0, gql_1.generateQuery)({
                queries: [
                    (0, gql_1.generateQueryData)({
                        tableName: table,
                        returning,
                        variables: Object.assign(Object.assign({}, variables), { where })
                    }),
                ],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0 });
        });
    }
    ;
    insert(objects = {}, options) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const _objects = Object.prototype.toString.call(objects) === '[object Array]' ? objects : [objects];
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.insertReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultInsertName;
            const q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                actions: [(0, gql_1.insertMutation)(table, Object.assign(Object.assign({}, variables), { objects: objects }), { tableName: table, operation: 'insert', returning })],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_c = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.m0) === null || _c === void 0 ? void 0 : _c.returning });
        });
    }
    ;
    update(exp, value, options) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : exp : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.updateReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultUpdateName;
            const q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                actions: [(0, gql_1.updateMutation)(table, Object.assign(Object.assign({}, variables), { where: exp, _set: value }), { tableName: table, operation: 'update', returning })],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_c = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.m0) === null || _c === void 0 ? void 0 : _c.returning });
        });
    }
    ;
    delete(exp, options) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : exp : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.deleteReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultDeleteName;
            const q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                actions: [(0, gql_1.deleteMutation)(table, Object.assign(Object.assign({}, variables), { where: exp, returning }), { tableName: table, operation: 'delete', returning })],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_c = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.m0) === null || _c === void 0 ? void 0 : _c.returning });
        });
    }
    ;
    reserve(count) {
        return (0, reserve_1.reserve)({ count, client: this.apolloClient });
    }
    ;
    await(id) {
        return (0, promise_1.awaitPromise)({ id, client: this.apolloClient });
    }
    ;
}
exports.DeepClient = DeepClient;
//# sourceMappingURL=client.js.map